﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Signal_Process;                                   // FFTに必要
using System.IO;                                        // ファイル操作に必要
using WAVE_file;                                        // waveファイルの操作に必要

namespace 鳥の声で種識別
{
    public partial class FFTtestForm : Form
    {
        // グローバル変数を宣言
        wave wave_file = new wave();
        
        
        public FFTtestForm()
        {
            InitializeComponent();
        }

        /*------------------------------------------
         * FFT実行ボタンが押された時の動作
         *  タイマーを動かしたり、停止させる
         * --------------------------------------------*/
        private void FFT_button_Click(object sender, EventArgs e)
        {
            if (this.FFTtimer.Enabled == true) {
                this.FFTtimer.Enabled = false;
                this.TimerStatusLabel.Text = "タイマー停止中";
            }else if (this.FFTtimer.Enabled == false){
                this.FFTtimer.Enabled = true;
                this.TimerStatusLabel.Text = "タイマー起動中";
            }
        }

        /*------------------------------------------
         * waveファイル選択ボタンが押された時の動作
         *  ファイルを選択し、可能なら開く
         * --------------------------------------------*/
        private void wave_set_button_Click(object sender, EventArgs e)
        {
            if (openFileDialog1.ShowDialog() == DialogResult.OK) {
                this.wave_file.Open(openFileDialog1);
            }
        }
        /*------------------------------------------
         * タイマーで定期的に実行される
         * 
         * --------------------------------------------*/
        private void FFTtimer_Tick(object sender, EventArgs e)
        {
            Graphics g = pictureBox1.CreateGraphics();                              // 図の描画の準備
            Pen blackpen = new Pen(Color.Black, 1);
            Pen redpen = new Pen(Color.Red, 1);
            Pen bluepen = new Pen(Color.Blue, 1);
            Pen greenpen = new Pen(Color.Green, 1);
            Pen orangepen = new Pen(Color.Orange, 1);
            Pen violetpen = new Pen(Color.Violet, 1);
            int x1, x2, y1, y2;
            DFT a = new DFT(DFT.FFT_point.size256 , DFT.Window.Perzen);             // FFTの準備
            DFT b = new DFT(DFT.FFT_point.size4096, DFT.Window.Perzen);             // FFTの準備
            DFT c = new DFT(DFT.FFT_point.size2048, DFT.Window.Perzen);             // FFTの準備
            DFT d = new DFT(DFT.FFT_point.size8192, DFT.Window.Perzen);             // FFTの準備
            DFT f = new DFT(DFT.FFT_point.size512 , DFT.Window.Perzen);             // FFTの準備
            const double read_size_sec = 0.1;                                       // waveファイルからの読み込み量[s]を指定
            double[] temp;
            int i;
            wave.MusicDatas[] data;                                                 // 音声ファイルから読み込んだデータ列が格納される。


            if (this.wave_file.IsOpen && this.wave_file.ReadLimit == false)         // 読めるなら、読み込む
            {
                // read
                data = this.wave_file.Read(read_size_sec);                          // クラスメソッドを使用してファイルからread_size_sec秒間読み込み。実行の度に更新される。
                temp = new double[this.wave_file.GetArrSize(read_size_sec)];        // FFT用に値を格納しなおす配列のインスタンスを生成
                for (i = 0; i < temp.Length; i++)
                {
                    temp[i] = (double)data[i].Right;                                // データをコピー
                }
                
                // view
                pictureBox1.Refresh();
                g.DrawLine(blackpen, 0, pictureBox1.Height / 2, pictureBox1.Width, pictureBox1.Height / 2); // 軸を書く
                x2 = y2 = 0;
                for (i = 0; i < temp.Length - 1; i++)                               // 読み込んだデータを描画
                {
                    x1 = x2;
                    y1 = y2;
                    x2 = (i + 1) * pictureBox1.Width / temp.Length;
                    y2 = (int)(-(pictureBox1.Height / 2) * temp[i + 1] / (2 << this.wave_file.Resolution - 1) + pictureBox1.Height / 2);
                    g.DrawLine(blackpen, x1, y1, x2, y2);
                }

                // FFT
                a.FFT(temp);                                                        // FFTを実行
                b.FFT(temp);                                                        // FFTサイズが音声データよりも小さい場合、始めの方しか処理しないので注意
                c.FFT(temp);
                d.FFT(temp);
                f.FFT(temp);

                // view d
                if (this.fft8192check.Checked)
                {
                    x2 = 0;
                    y2 = (int)(-(pictureBox1.Height / 2) * d.calced_datas[0].Log / 5 + pictureBox1.Height);
                    for (i = 1; i < d.FFTsize / 2 - 1; i += 4)                      // リアルパートのみ表示
                    {
                        x1 = x2;
                        y1 = y2;
                        x2 = i * pictureBox1.Width * 2 / d.FFTsize;                 // FFT実部を描画。
                        y2 = (int)(-(pictureBox1.Height / 2) * d.calced_datas[i].Log / 5 + pictureBox1.Height);
                        if (y1 >= 0 && y2 >= 0) g.DrawLine(bluepen, x1, y1, x2, y2);
                    }
                }
                // view b
                if (this.fft4096check.Checked)
                {
                    x2 = 0;
                    y2 = (int)(-(pictureBox1.Height / 2) * b.calced_datas[0].Log / 5 + pictureBox1.Height);
                    for (i = 1; i < b.FFTsize / 2 - 1; i += 3)                      // リアルパートのみ表示
                    {
                        x1 = x2;
                        y1 = y2;
                        x2 = i * pictureBox1.Width * 2 / b.FFTsize;                 // FFT実部を描画。
                        y2 = (int)(-(pictureBox1.Height / 2) * b.calced_datas[i].Log / 5 + pictureBox1.Height);
                        if(y1 >= 0 && y2 >= 0) g.DrawLine(greenpen, x1, y1, x2, y2);
                    }
                }
                // view c
                if (this.fft2048check.Checked)
                {
                    x2 = 0;
                    y2 = (int)(-(pictureBox1.Height / 2) * c.calced_datas[0].Log / 5 + pictureBox1.Height);
                    for (i = 1; i < c.FFTsize / 2 - 1; i += 2)                      // リアルパートのみ表示
                    {
                        x1 = x2;
                        y1 = y2;
                        x2 = i * pictureBox1.Width * 2 / c.FFTsize;                 // FFT実部を描画。
                        y2 = (int)(-(pictureBox1.Height / 2) * c.calced_datas[i].Log / 5 + pictureBox1.Height);
                        if (y1 >= 0 && y2 >= 0) g.DrawLine(violetpen, x1, y1, x2, y2);
                    }
                }
                // view e
                if (this.fft512check.Checked)
                {
                    x2 = 0;
                    y2 = (int)(-(pictureBox1.Height / 2) * d.calced_datas[0].Log / 5 + pictureBox1.Height);
                    for (i = 1; i < f.FFTsize / 2 - 1; i++)                         // リアルパートのみ表示
                    {
                        x1 = x2;
                        y1 = y2;
                        x2 = i * pictureBox1.Width * 2 / f.FFTsize;                 // FFT実部を描画。
                        y2 = (int)(-(pictureBox1.Height / 2) * f.calced_datas[i].Log / 5 + pictureBox1.Height);
                        if (y1 >= 0 && y2 >= 0) g.DrawLine(orangepen, x1, y1, x2, y2);
                    }
                }
                // view a
                if (this.fft256check.Checked)
                {
                    x2 = 0;
                    y2 = (int)(-(pictureBox1.Height / 2) * a.calced_datas[0].Log / 5 + pictureBox1.Height);
                    for (i = 1; i < a.FFTsize / 2 - 1; i++)                         // リアルパートのみ表示
                    {
                        x1 = x2;
                        y1 = y2;
                        x2 = i * pictureBox1.Width * 2 / a.FFTsize;                 // FFT実部を描画。
                        y2 = (int)(-(pictureBox1.Height / 2) * a.calced_datas[i].Log / 5 + pictureBox1.Height);
                        if (y1 >= 0 && y2 >= 0) g.DrawLine(redpen, x1, y1, x2, y2);
                    }
                }

                this.TimeAtReadLabel.Text = "読み出し時刻 = " + this.wave_file.NowTime + "[s]";
            }
            g.Dispose();
        }
    }
}
